%% Sensor placement analysis of the model in 
%  "Structural analysis for the sensor location problem
%  in fault detection and isolation" by C. Commault, 
%  J. Dion and S.Y. Agha, Proceedings of Safeprocess'06 
%  Beijing, China.
%
%  To compare results, remember the difference that in this
%  approach, in contrast to the Commault et.al. paper, 
%  the fault signals are not considered measurable.

%% Define model structure and define structural model object.
clear
modelDef.type = 'MatrixStruc';
modelDef.X = [0 0 0 0 1 0 0;... 
     0 0 0 0 0 0 1;... 
     0 0 0 0 0 1 0;... 
     1 0 0 0 1 0 1;...
     0 1 0 0 0 1 0;... 
     1 1 1 0 0 0 0;...     
     1 1 0 1 0 0 0;...
     0 0 1 0 0 0 0;... 
     0 0 1 1 0 0 0;... 
     0 0 0 1 0 0 0];   

modelDef.F = [1 0 0 0;...
     0 1 0 0;...
     0 0 0 1;...
     0 0 0 0;...
     0 0 1 0;...
     0 0 0 0;...
     0 0 0 0;...
     0 0 0 0;...
     0 0 0 0;...
     0 0 0 0];

modelDef.Z = [0 0 0;...
     0 0 0;...
     0 0 0;...
     0 0 0;...
     0 0 0;...
     0 0 0;...
     0 0 0;...
     1 0 0;...
     0 1 0;...
     0 0 1];

model = DiagnosisModel(modelDef);
model.name = 'The example from Commault et.al.';
clear modelDef

%% Plot model structure
figure(1)
model.PlotModel();

%% Perform isolability analytsis without adding any new sensors
figure(2)
model.IsolabilityAnalysis();

%% Perform sensor placement analysis, add first sensor set, and redo
% the isolability analysis.
s = model.SensorPlacementIsolability();
model2 = model.AddSensors(s{1});
figure(3)
model2.IsolabilityAnalysis();

